@@ -11,10 +11,7 @@ use thiserror::Error;
 
 pub mod polynomial;
 
-use polynomial::{
-    Polynomial,
-    op_tree
-};
+use polynomial::{op_tree, Polynomial};
 
 /// parameters from tested setup
 #[derive(Clone, Debug)]
@@ -61,9 +58,9 @@ impl<E: Engine> KZGWitness<E> {
 
 // A witness for a several elements - "w_B" in the paper. It's a single group element plus a polynomial
 #[derive(Debug, Clone, PartialEq, Eq)]
-pub struct KZGBatchWitness<E: Engine, const MAX_COEFFS: usize>{
+pub struct KZGBatchWitness<E: Engine, const MAX_COEFFS: usize> {
     r: Polynomial<E, MAX_COEFFS>,
-    w: E::G1Affine
+    w: E::G1Affine,
 }
 
 impl<E: Engine, const MAX_COEFFS: usize> KZGBatchWitness<E, MAX_COEFFS> {
@@ -112,21 +109,18 @@ impl<'params, E: Engine, const MAX_COEFFS: usize> KZGProver<'params, E, MAX_COEF
         }
     }
 
+    pub fn parameters(&self) -> &'params KZGParams<E, MAX_COEFFS> {
+        self.parameters
+    }
+
     pub fn commit(&mut self, polynomial: Polynomial<E, MAX_COEFFS>) -> KZGCommitment<E> {
-        let commitment = op_tree(
-            polynomial.num_coeffs(),
-            &|i| {
-                if i == 0 {
-                    self.parameters.g * polynomial.coeffs[i]
-                } else {
-                    self.parameters.gs[i - 1] * polynomial.coeffs[i]
-                }
-            },
-            &|a, b| a + b
-        ).to_affine();
+        let mut commitment = self.parameters.g * polynomial.coeffs[0];
+        for i in 0..polynomial.degree() {
+            commitment += self.parameters.gs[i] * polynomial.coeffs[i + 1];
+        }
 
         self.polynomial = Some(polynomial);
-        let commitment = KZGCommitment(commitment);
+        let commitment = KZGCommitment(commitment.to_affine());
         self.commitment = Some(commitment);
         commitment
     }
@@ -170,19 +164,12 @@ impl<'params, E: Engine, const MAX_COEFFS: usize> KZGProver<'params, E, MAX_COEF
                     // self.polynomial(point.y) != point.1
                     (_, Some(_)) => Err(KZGError::PointNotOnPolynomial),
                     (psi, None) => {
-                        let witness = op_tree(
-                            psi.num_coeffs(), 
-                            &|i| {
-                                if i == 0 {
-                                    self.parameters.g * psi.coeffs[i]
-                                } else {
-                                    self.parameters.gs[i - 1] * psi.coeffs[i]
-                                }
-                            },
-                            &|a, b| a + b
-                        ).to_affine(); 
+                        let mut w = self.parameters.g * psi.coeffs[0];
+                        for i in 0..psi.degree() {
+                            w += self.parameters.gs[i] * psi.coeffs[i + 1];
+                        }
 
-                        Ok(KZGWitness(witness))
+                        Ok(KZGWitness(w.to_affine()))
                     }
                 }
             }
@@ -190,7 +177,10 @@ impl<'params, E: Engine, const MAX_COEFFS: usize> KZGProver<'params, E, MAX_COEF
     }
 
     // #[cfg(any(std))]
-    pub fn create_witness_batched(&mut self, points: &Vec<(E::Fr, E::Fr)>) -> Result<KZGBatchWitness<E, MAX_COEFFS>, KZGError> {
+    pub fn create_witness_batched(
+        &mut self,
+        points: &Vec<(E::Fr, E::Fr)>,
+    ) -> Result<KZGBatchWitness<E, MAX_COEFFS>, KZGError> {
         match self.polynomial {
             None => Err(KZGError::NoPolynomial),
             Some(ref polynomial) => {
@@ -202,29 +192,28 @@ impl<'params, E: Engine, const MAX_COEFFS: usize> KZGProver<'params, E, MAX_COEF
                         coeffs[1] = E::Fr::one();
                         Polynomial::new_from_coeffs(coeffs, 1)
                     },
-                    &|a, b| a * b
+                    &|a, b| a * b,
                 );
-                
+
                 let (xs, ys): (Vec<E::Fr>, Vec<E::Fr>) = points.iter().cloned().unzip();
-                let interpolation = Polynomial::<E, MAX_COEFFS>::lagrange_interpolation(xs.as_slice(), ys.as_slice());
+                let interpolation = Polynomial::<E, MAX_COEFFS>::lagrange_interpolation(
+                    xs.as_slice(),
+                    ys.as_slice(),
+                );
 
                 let numerator = polynomial - &interpolation;
                 let (psi, rem) = numerator.long_division(&zeros);
                 match rem {
                     Some(_) => Err(KZGError::PointNotOnPolynomial),
                     None => {
-                        let w = op_tree(
-                            psi.num_coeffs(),
-                            &|i| {
-                                if i == 0 {
-                                    self.parameters.g * psi.coeffs[i]
-                                } else {
-                                    self.parameters.gs[i - 1] * psi.coeffs[i]
-                                }
-                            },
-                            &|a, b| a + b
-                        ).to_affine();
-                        Ok(KZGBatchWitness { r: interpolation, w })
+                        let mut w = self.parameters.g * psi.coeffs[0];
+                        for i in 0..psi.degree() {
+                            w += self.parameters.gs[i] * psi.coeffs[i + 1];
+                        }
+                        Ok(KZGBatchWitness {
+                            r: interpolation,
+                            w: w.to_affine(),
+                        })
                     }
                 }
             }
@@ -242,17 +231,10 @@ impl<'params, E: Engine, const MAX_COEFFS: usize> KZGVerifier<'params, E, MAX_CO
         commitment: &KZGCommitment<E>,
         polynomial: &Polynomial<E, MAX_COEFFS>,
     ) -> bool {
-        let check = op_tree(
-            polynomial.num_coeffs(), 
-            &|i| {
-                if i == 0 {
-                    self.parameters.g * polynomial.coeffs[i]
-                } else {
-                    self.parameters.gs[i - 1] * polynomial.coeffs[i]
-                }
-            },
-            &|a, b| a + b
-        );
+        let mut check = self.parameters.g * polynomial.coeffs[0];
+        for i in 0..polynomial.degree() {
+            check += self.parameters.gs[i] * polynomial.coeffs[i + 1];
+        }
 
         check.to_affine() == commitment.0
     }
@@ -279,7 +261,7 @@ impl<'params, E: Engine, const MAX_COEFFS: usize> KZGVerifier<'params, E, MAX_CO
         &self,
         points: &Vec<(E::Fr, E::Fr)>,
         commitment: &KZGCommitment<E>,
-        witness: &KZGBatchWitness<E, MAX_COEFFS>
+        witness: &KZGBatchWitness<E, MAX_COEFFS>,
     ) -> bool {
         let z: Polynomial<E, MAX_COEFFS> = op_tree(
             points.len(),
@@ -289,37 +271,23 @@ impl<'params, E: Engine, const MAX_COEFFS: usize> KZGVerifier<'params, E, MAX_CO
                 coeffs[1] = E::Fr::one();
                 Polynomial::new_from_coeffs(coeffs, 1)
             },
-            &|a, b| a * b
+            &|a, b| a * b,
         );
 
-        let hz = op_tree(
-            z.num_coeffs(),
-            &|i| {
-                if i == 0 {
-                    self.parameters.h * z.coeffs[i]
-                } else {
-                    self.parameters.hs[i - 1] * z.coeffs[i]
-                }
-            },
-            &|a, b| a + b
-        ).to_affine();
+        let mut hz = self.parameters.h * z.coeffs[0];
+        for i in 0..z.degree() {
+            hz += self.parameters.hs[i] * z.coeffs[i + 1];
+        }
 
-        let gr = op_tree(
-            witness.r.num_coeffs(),
-            &|i| {
-                if i == 0 {
-                    self.parameters.g * witness.r.coeffs[i]
-                } else {
-                    self.parameters.gs[i - 1] * witness.r.coeffs[i]
-                }
-            },
-            &|a, b| a + b
-        );
+        let mut gr = self.parameters.g * witness.r.coeffs[0];
+        for i in 0..witness.r.degree() {
+            gr += self.parameters.gs[i] * witness.r.coeffs[i + 1];
+        }
 
-        let lhs = E::pairing(&witness.w, &hz);
+        let lhs = E::pairing(&witness.w, &hz.to_affine());
         let rhs = E::pairing(
             &(commitment.0.to_curve() - gr).to_affine(),
-            &self.parameters.h
+            &self.parameters.h,
         );
 
         lhs == rhs
@@ -379,8 +347,11 @@ mod tests {
         setup(s)
     }
 
-    fn test_participants<'params, E: Engine, const MAX_COEFFS: usize>(params: &'params KZGParams<E, MAX_COEFFS>) -> (
-        KZGProver<'params, E, MAX_COEFFS>, KZGVerifier<'params, E, MAX_COEFFS>
+    fn test_participants<'params, E: Engine, const MAX_COEFFS: usize>(
+        params: &'params KZGParams<E, MAX_COEFFS>,
+    ) -> (
+        KZGProver<'params, E, MAX_COEFFS>,
+        KZGVerifier<'params, E, MAX_COEFFS>,
     ) {
         let prover = KZGProver::new(params);
         let verifier = KZGVerifier::new(params);
