@@ -160,6 +160,13 @@ impl<E: Engine, const MAX_COEFFS: usize> Polynomial<E, MAX_COEFFS> {
     pub fn lagrange_interpolation(xs: &[E::Fr], ys: &[E::Fr]) -> Polynomial<E, MAX_COEFFS> {
         assert_eq!(xs.len(), ys.len());
 
+        // handle trivial case where there's only 1 point
+        if xs.len() == 1 {
+            let mut coeffs = [E::Fr::zero(); MAX_COEFFS];
+            coeffs[0] = ys[0];
+            return Polynomial::new_from_coeffs(coeffs, 0);
+        }
+        
         op_tree(
             xs.len(),
             &|i| {
